@@ -1,16 +1,17 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0+ OR GPL-3.0 WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "jsonsummarypage.h"
 
 #include "jsonwizard.h"
+#include "../buildsystem.h"
 #include "../project.h"
 #include "../projectexplorerconstants.h"
+#include "../projectexplorertr.h"
 #include "../projectnodes.h"
+#include "../projectmanager.h"
 #include "../projecttree.h"
-#include "../session.h"
-
-#include "../projecttree.h"
+#include "../target.h"
 
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/iversioncontrol.h>
@@ -27,7 +28,6 @@ using namespace Utils;
 
 static char KEY_SELECTED_PROJECT[] = "SelectedProject";
 static char KEY_SELECTED_NODE[] = "SelectedFolderNode";
-static char KEY_IS_SUBPROJECT[] = "IsSubproject";
 static char KEY_VERSIONCONTROL[] = "VersionControl";
 static char KEY_QT_KEYWORDS_ENABLED[] = "QtKeywordsEnabled";
 
@@ -63,12 +63,12 @@ static IWizardFactory::WizardKind wizardKind(JsonWizard *wiz)
 // --------------------------------------------------------------------
 
 JsonSummaryPage::JsonSummaryPage(QWidget *parent) :
-    Internal::ProjectWizardPage(parent),
+    ProjectWizardPage(parent),
     m_wizard(nullptr)
 {
-    connect(this, &Internal::ProjectWizardPage::projectNodeChanged,
+    connect(this, &ProjectWizardPage::projectNodeChanged,
             this, &JsonSummaryPage::summarySettingsHaveChanged);
-    connect(this, &Internal::ProjectWizardPage::versionControlChanged,
+    connect(this, &ProjectWizardPage::versionControlChanged,
             this, &JsonSummaryPage::summarySettingsHaveChanged);
 }
 
@@ -77,6 +77,24 @@ void JsonSummaryPage::setHideProjectUiValue(const QVariant &hideProjectUiValue)
     m_hideProjectUiValue = hideProjectUiValue;
 }
 
+static Node *extractPreferredNode(const JsonWizard *wizard)
+{
+    // Use static cast from void * to avoid qobject_cast (which needs a valid object) in value()
+    // in the following code:
+    Node *preferred = nullptr;
+    QVariant variant = wizard->value(Constants::PREFERRED_PROJECT_NODE);
+    if (variant.isValid()) {
+        preferred = static_cast<Node *>(variant.value<void *>());
+    } else {
+        variant = wizard->value(Constants::PREFERRED_PROJECT_NODE_PATH);
+        if (variant.isValid()) {
+            const FilePath fp = FilePath::fromVariant(variant);
+            preferred = ProjectTree::instance()->nodeForFile(fp);
+        }
+    }
+    return preferred;
+}
+
 void JsonSummaryPage::initializePage()
 {
     m_wizard = qobject_cast<JsonWizard *>(wizard());
@@ -84,14 +102,20 @@ void JsonSummaryPage::initializePage()
 
     m_wizard->setValue(QLatin1String(KEY_SELECTED_PROJECT), QVariant());
     m_wizard->setValue(QLatin1String(KEY_SELECTED_NODE), QVariant());
-    m_wizard->setValue(QLatin1String(KEY_IS_SUBPROJECT), false);
     m_wizard->setValue(QLatin1String(KEY_VERSIONCONTROL), QString());
     m_wizard->setValue(QLatin1String(KEY_QT_KEYWORDS_ENABLED), false);
 
     connect(m_wizard, &JsonWizard::filesReady, this, &JsonSummaryPage::triggerCommit);
     connect(m_wizard, &JsonWizard::filesReady, this, &JsonSummaryPage::addToProject);
 
+    // set result to accepted, so we can check if updateFileList -> m_wizard->generateFileList
+    // called reject()
+    m_wizard->setResult(QDialog::Accepted);
     updateFileList();
+    // if there were errors while updating the file list, the dialog is rejected
+    // so don't continue the setup (which also avoids showing the error message again)
+    if (m_wizard->result() == QDialog::Rejected)
+        return;
 
     IWizardFactory::WizardKind kind = wizardKind(m_wizard);
     bool isProject = (kind == IWizardFactory::ProjectWizard);
@@ -110,23 +134,43 @@ void JsonSummaryPage::initializePage()
                                  });
     }
 
-    // Use static cast from void * to avoid qobject_cast (which needs a valid object) in value()
-    // in the following code:
-    auto contextNode = findWizardContextNode(static_cast<Node *>(m_wizard->value(Constants::PREFERRED_PROJECT_NODE).value<void *>()));
+    Node *preferredNode = extractPreferredNode(m_wizard);
+    const FilePath preferredNodePath = preferredNode ? preferredNode->filePath() : FilePath{};
+    auto contextNode = findWizardContextNode(preferredNode);
     const ProjectAction currentAction = isProject ? AddSubProject : AddNewFile;
+    const bool isSubproject = m_wizard->value(Constants::PROJECT_ISSUBPROJECT).toBool();
 
-    initializeProjectTree(contextNode, files, kind, currentAction);
+    const auto updateProjectTree = [this, files, kind, currentAction, preferredNodePath] {
+        Node *node = currentNode();
+        if (!node) {
+            if (auto p = ProjectManager::projectWithProjectFilePath(preferredNodePath))
+                node = p->rootProjectNode();
+        }
+        initializeProjectTree(findWizardContextNode(node), files, kind, currentAction,
+                              m_wizard->value(Constants::PROJECT_ISSUBPROJECT).toBool());
+    };
+
+    if (contextNode) {
+        if (auto p = contextNode->getProject()) {
+            if (auto targets = p->targets(); !targets.isEmpty()) {
+                if (auto bs = targets.first()->buildSystem()) {
+                    if (bs->isParsing()) {
+                        connect(bs, &BuildSystem::parsingFinished, this, updateProjectTree,
+                                Qt::SingleShotConnection);
+                    }
+                }
+            }
+        }
+    }
+    initializeProjectTree(contextNode, files, kind, currentAction, isSubproject);
 
     // Refresh combobox on project tree changes:
-    connect(ProjectTree::instance(), &ProjectTree::treeChanged,
-            this, [this, files, kind, currentAction]() {
-        initializeProjectTree(findWizardContextNode(currentNode()), files, kind, currentAction);
-    });
-
+    connect(ProjectTree::instance(), &ProjectTree::treeChanged, this, updateProjectTree);
 
     bool hideProjectUi = JsonWizard::boolFromVariant(m_hideProjectUiValue, m_wizard->expander());
     setProjectUiVisible(!hideProjectUi);
 
+    setVersionControlUiElementsVisible(!isSubproject);
     initializeVersionControls();
 
     // Do a new try at initialization, now that we have real values set up:
@@ -152,8 +196,8 @@ void JsonSummaryPage::triggerCommit(const JsonWizard::GeneratorFiles &files)
 
     QString errorMessage;
     if (!runVersionControl(coreFiles, &errorMessage)) {
-        QMessageBox::critical(wizard(), tr("Failed to Commit to Version Control"),
-                              tr("Error message from Version Control System: \"%1\".")
+        QMessageBox::critical(wizard(), Tr::tr("Failed to Commit to Version Control"),
+                              Tr::tr("Error message from Version Control System: \"%1\".")
                               .arg(errorMessage));
     }
 }
@@ -169,8 +213,8 @@ void JsonSummaryPage::addToProject(const JsonWizard::GeneratorFiles &files)
         return;
     if (kind == IWizardFactory::ProjectWizard) {
         if (!static_cast<ProjectNode *>(folder)->addSubProject(generatedProject)) {
-            QMessageBox::critical(m_wizard, tr("Failed to Add to Project"),
-                                  tr("Failed to add subproject \"%1\"\nto project \"%2\".")
+            QMessageBox::critical(m_wizard, Tr::tr("Failed to Add to Project"),
+                                  Tr::tr("Failed to add subproject \"%1\"\nto project \"%2\".")
                                   .arg(generatedProject.toUserOutput())
                                   .arg(folder->filePath().toUserOutput()));
             return;
@@ -181,8 +225,8 @@ void JsonSummaryPage::addToProject(const JsonWizard::GeneratorFiles &files)
             return f.file.filePath();
         });
         if (!folder->addFiles(filePaths)) {
-            QMessageBox::critical(wizard(), tr("Failed to Add to Project"),
-                                  tr("Failed to add one or more files to project\n\"%1\" (%2).")
+            QMessageBox::critical(wizard(), Tr::tr("Failed to Add to Project"),
+                                  Tr::tr("Failed to add one or more files to project\n\"%1\" (%2).")
                                   .arg(folder->filePath().toUserOutput(),
                                        FilePath::formatFilePaths(filePaths, ", ")));
             return;
@@ -210,10 +254,10 @@ Node *JsonSummaryPage::findWizardContextNode(Node *contextNode) const
 
         // Static cast from void * to avoid qobject_cast (which needs a valid object) in value().
         auto project = static_cast<Project *>(m_wizard->value(Constants::PROJECT_POINTER).value<void *>());
-        if (SessionManager::projects().contains(project) && project->rootProjectNode()) {
-            const QString path = m_wizard->value(Constants::PREFERRED_PROJECT_NODE_PATH).toString();
+        if (ProjectManager::projects().contains(project) && project->rootProjectNode()) {
+            const FilePath path = FilePath::fromVariant(m_wizard->value(Constants::PREFERRED_PROJECT_NODE_PATH));
             contextNode = project->rootProjectNode()->findNode([path](const Node *n) {
-                return path == n->filePath().toString();
+                return path == n->filePath();
             });
         }
     }
@@ -235,7 +279,7 @@ void JsonSummaryPage::updateProjectData(FolderNode *node)
 
     m_wizard->setValue(QLatin1String(KEY_SELECTED_PROJECT), QVariant::fromValue(project));
     m_wizard->setValue(QLatin1String(KEY_SELECTED_NODE), QVariant::fromValue(node));
-    m_wizard->setValue(QLatin1String(KEY_IS_SUBPROJECT), node ? true : false);
+    m_wizard->setValue(QLatin1String(Constants::PROJECT_ISSUBPROJECT), node ? true : false);
     bool qtKeyWordsEnabled = true;
     if (ProjectTree::hasNode(node)) {
         const ProjectNode *projectNode = node->asProjectNode();
@@ -255,6 +299,19 @@ void JsonSummaryPage::updateProjectData(FolderNode *node)
     m_wizard->setValue(QLatin1String(KEY_QT_KEYWORDS_ENABLED), qtKeyWordsEnabled);
 
     updateFileList();
+    setStatusVisible(false);
+    if (wizardKind(m_wizard) != IWizardFactory::ProjectWizard)
+        return;
+    if (node && !m_fileList.isEmpty()) {
+        const FilePath parentFolder = node->directory();
+        const FilePath subProjectFolder = m_fileList.first().file.filePath().parentDir();
+        if (!subProjectFolder.isChildOf(parentFolder)) {
+            setStatus(Tr::tr("Subproject \"%1\" outside of \"%2\".")
+                      .arg(subProjectFolder.toUserOutput()).arg(parentFolder.toUserOutput()),
+                      InfoLabel::Warning);
+            setStatusVisible(true);
+        }
+    }
 }
 
 } // namespace ProjectExplorer
